
class Pizza:
    def __init__(self):
        self._dough = ""
        self._sauce = ""
        self._topping = ""

    def set_dough(self, dough):
        self._dough = dough

    def set_sauce(self, sauce):
        self._sauce = sauce

    def set_topping(self, topping):
        self._topping = topping

    def open(self):
        print(f"Pizza with {self._dough} dough, {self._sauce} sauce and {self._topping} topping. Mmm.")


class PizzaBuilder:
    def __init__(self):
        self._pizza = Pizza()

    def get_pizza(self):
        return self._pizza

    def create_new_pizza_product(self):
        self._pizza = Pizza()

    def build_dough(self):
        pass

    def build_sauce(self):
        pass

    def build_topping(self):
        pass


class HawaiianPizzaBuilder(PizzaBuilder):
    def build_dough(self):
        self._pizza.set_dough("cross")

    def build_sauce(self):
        self._pizza.set_sauce("mild")

    def build_topping(self):
        self._pizza.set_topping("ham and pineapple")


class SpicyPizzaBuilder(PizzaBuilder):
    def build_dough(self):
        self._pizza.set_dough("pan baked")

    def build_sauce(self):
        self._pizza.set_sauce("hot")

    def build_topping(self):
        self._pizza.set_topping("pepperoni and salami")


class Cook:
    def __init__(self):
        self._pizza_builder = None

    def set_pizza_builder(self, pb):
        self._pizza_builder = pb

    def construct_pizza(self):
        self._pizza_builder.create_new_pizza_product()
        self._pizza_builder.build_dough()
        self._pizza_builder.build_sauce()
        self._pizza_builder.build_topping()

    def get_pizza(self):
        return self._pizza_builder.get_pizza()


def main():
    cook = Cook()
    hawaiian_pizza_builder = HawaiianPizzaBuilder()
    spicy_pizza_builder = SpicyPizzaBuilder()

    cook.set_pizza_builder(hawaiian_pizza_builder)
    cook.construct_pizza()

    hawaiian_pizza = cook.get_pizza()
    hawaiian_pizza.open()

    cook.set_pizza_builder(spicy_pizza_builder)
    cook.construct_pizza()

    spicy_pizza = cook.get_pizza()
    spicy_pizza.open()

if __name__ == '__main__':
    main()
